@@ -179,51 +179,44 @@ sub make {
             # email if the address is not set at loop above.
             $p->{'addresser'} ||= $messageobj->{'header'}->{'to'}; 
 
-            # if( length $p->{'recipient'} == 0 ) {
-            #    # Detect "recipient" address if it is not set yet
-            #    for my $f ( @{ $fieldorder->{'recipient'} } ) {
-            #        # Check each header in message/rfc822 part
-            #        my $h = lc $f;
-            #        next unless exists $rfc822data->{ $h };
-            #        next unless length $rfc822data->{ $h };
-            #        next unless Sisimai::RFC5322->is_emailaddress( $rfc822data->{ $h } );
-            #        $p->{'recipient'} = $rfc822data->{ $h };
-            #        last;
-            #    }
-            # }
-
         } # End of EMAIL_ADDRESS
         next unless $p->{'addresser'};
         next unless $p->{'recipient'};
 
         TIMESTAMP: {
             # Convert from a time stamp or a date string to a machine time.
-            my $v = $e->{'date'} || '';
+            my $datestring = undef;
+            my $zoneoffset = 0;
+            my @datevalues = ();
 
-            unless( $v ) {
-                # Date information did not exist in message/delivery-status part,...
-                for my $f ( @{ Sisimai::MTA->RFC822HEADERS('date') } ) {
-                    # Get the value of Date header or other date related header.
-                    next unless $rfc822data->{ lc $f };
-                    $v = $rfc822data->{ lc $f };
-                    last;
-                }
+            push @datevalues, $e->{'date'} if $e->{'date'};
 
-                unless( $v ) {
-                    # Set "date" getting from the value of "Date" in the bounce
-                    # message
-                    $v= $messageobj->{'header'}->{'date'}; 
-                }
+            # Date information did not exist in message/delivery-status part,...
+            for my $f ( @{ Sisimai::MTA->RFC822HEADERS('date') } ) {
+                # Get the value of Date header or other date related header.
+                next unless $rfc822data->{ lc $f };
+                push @datevalues, $rfc822data->{ lc $f };
             }
 
-            my $datestring = Sisimai::Time->parse( $v );
-            my $zoneoffset = 0;
+            if( scalar( @datevalues ) < 2 ) {
+                # Set "date" getting from the value of "Date" in the bounce message
+                push @datevalues, $messageobj->{'header'}->{'date'}; 
+            }
 
-            if( $datestring =~ m/\A(.+)\s+([-+]\d{4})\z/ ) {
-                # Wed, 26 Feb 2014 06:05:48 -0500
-                $datestring = $1;
-                $zoneoffset = Sisimai::Time->tz2second($2);
-                $p->{'timezoneoffset'} = $2;
+            while( my $v = shift @datevalues ) {
+                # Parse each date value in the array
+                $datestring = Sisimai::Time->parse( $v );
+                last if $datestring;
+            }
+
+            if( defined $datestring ) {
+                # Get the value of timezone offset from $datestring
+                if( $datestring =~ m/\A(.+)\s+([-+]\d{4})\z/ ) {
+                    # Wed, 26 Feb 2014 06:05:48 -0500
+                    $datestring = $1;
+                    $zoneoffset = Sisimai::Time->tz2second($2);
+                    $p->{'timezoneoffset'} = $2;
+                }
             }
 
             eval {
