@@ -5,10 +5,14 @@ use warnings;
 
 # http://tools.ietf.org/html/rfc3464
 my $RxRFC = {
-    'begin'  => qr{\A(?:
-         Content-Type:[ ]*message/delivery-status
+    'begin'  => qr{\A(?>
+         Content-Type:[ ]*(?:
+              message/delivery-status
+             |message/disposition-notification
+             )
         |The[ ]original[ ]message[ ]was[ ]received[ ]at[ ]
         |This[ ]report[ ]relates[ ]to[ ]your[ ]message
+        |Your[ ]message[ ]was[ ]not[ ]delivered[ ]to[ ]the[ ]following[ ]recipients
         )
     }xi,
     'endof'  => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
@@ -17,9 +21,29 @@ my $RxRFC = {
         |Return-Path:[ ]*[<].+[>]\z
         )\z
     }xi,
+    'error'  => qr{\A(?:
+         [45]\d\d\s+
+        |[<][^@]+[@][^@]+[>]:?\s+
+        )
+    }xi,
 };
 
-sub version     { '4.0.10' };
+# Regular expressions for BODY_PARSER_FOR_FALLBACK block.
+my $RxCmd = qr{[ ](RCPT|MAIL|DATA)[ ]+command\b};            # SMTP Command
+my $RxEMF = qr{\b(?:postmaster|mailer-daemon|root)[@]}i;    # From:
+my $RxEMR = qr{(?:[<][>]|mailer-daemon)}i;                  # Return-Path:
+my $RxEMS = qr{(?:                                          # Subject:
+     delivery[ ]failure
+    |Delivery[ ]Report
+    |failure[ ]notice
+    |mail[ ]error
+    |non[-]delivery
+    |returned[ ]mail
+    |undeliverable[ ]mail
+    )
+}xi;
+
+sub version     { '4.0.20' };
 sub description { 'Fallback Module for MTAs' };
 sub smtpagent   { 'RFC3464' };
 
@@ -31,6 +55,7 @@ sub scan {
     my $class = shift;
     my $mhead = shift // return undef;
     my $mbody = shift // return undef;
+    my $match = 0;
 
     return undef unless keys %$mhead;
     return undef unless ref $mbody eq 'SCALAR';
@@ -93,9 +118,10 @@ sub scan {
             # Before "message/rfc822"
             next unless ( $e =~ $RxRFC->{'begin'} ) .. ( $e =~ $RxRFC->{'rfc822'} );
             next unless length $e;
-  
+
             $v = $dscontents->[ -1 ];
-            if( $e =~ m/\A(?:[Ff]inal|[Oo]riginal)-[Rr]ecipient:[ ]*(?:RFC|rfc)822;[ ]*([^ ]+)\z/ ) {
+            if( $e =~ m/\A(?:[Ff]inal|[Oo]riginal)-[Rr]ecipient:[ ]*(?:RFC|rfc)822;[ ]*([^ ]+)\z/ ||
+                $e =~ m/\A(?:[Ff]inal|[Oo]riginal)-[Rr]ecipient:[ ]*([^ ]+)\z/ ) {
                 # 2.3.2 Final-Recipient field
                 #   The Final-Recipient field indicates the recipient for which this set
                 #   of per-recipient fields applies.  This field MUST be present in each
@@ -214,6 +240,11 @@ sub scan {
                     $v->{'spec'} = uc $1;
                     $v->{'diagnosis'} = $2;
 
+                } elsif( $e =~ m/\A[Dd]iagnostic-[Cc]ode:[ ]*(.+)\z/ ) {
+                    # No value of "diagnostic-type"
+                    # Diagnostic-Code: 554 ...
+                    $v->{'diagnosis'} = $1;
+
                 } elsif( $p =~ m/\A[Dd]iagnostic-[Cc]ode:[ ]*/ && $e =~ m/\A[\s\t]+(.+)\z/ ) {
                     # Continued line of the value of Diagnostic-Code header
                     $v->{'diagnosis'} .= ' '.$1;
@@ -264,7 +295,10 @@ sub scan {
                     } else {
                         # Get error message
                         next if $e =~ m/\A[ -]+/;
-                        next unless $e =~ m/\A[45]\d\d\s+/;
+                        next unless $e =~ $RxRFC->{'error'};
+
+                        # 500 User Unknown
+                        # <kijitora@example.jp> Unknown
                         $v->{'alterrors'} .= ' '.$e;
                     }
                 }
@@ -277,6 +311,108 @@ sub scan {
         $e = '';
     }
 
+    BODY_PARSER_FOR_FALLBACK: {
+        # Fallback, parse entire message body
+
+        unless( $recipients ) {
+            # Failed to get a recipient address at code above
+            $match = 1 if $mhead->{'from'}        =~ $RxEMF;
+            $match = 1 if $mhead->{'subject'}     =~ $RxEMS;
+            if( defined $mhead->{'return-path'} ) {
+                # Check the value of Return-Path of the message
+                $match = 1 if $mhead->{'return-path'} =~ $RxEMR;
+            }
+            last unless $match;
+
+            my $RxSkip = qr{(?>
+                 \A[-]+=
+                |\A\\\*
+                |\A[\s\t]+\z
+                |\A\s*--
+                |\AHi[ ][!]
+                |Content-(?:Description|Disposition|Transfer-Encoding|Type):[ ]
+                |(?:name|charset)=
+                |--\z
+                |:[ ]--------
+                )
+            }xi;
+
+            my $RxEnd  = qr{(?:
+                  \AContent-Type:[ ]message/delivery-status
+                 |\AHere[ ]is[ ]a[ ]copy[ ]of[ ]the[ ]first[ ]part[ ]of[ ]the[ ]message
+                 |\AThe[ ]non-delivered[ ]message[ ]is[ ]attached[ ]to[ ]this[ ]message.
+                 |\AReceived:\s*
+                 |\AReturn-Path:\s*
+                 |\AA[ ]copy[ ]of[ ]the[ ]original[ ]message[ ]below[ ]this[ ]line:
+                 |Attachment[ ]is[ ]a[ ]copy[ ]of[ ]the[ ]message
+                 |Below[ ]is[ ]a[ ]copy[ ]of[ ]the[ ]original[ ]message:
+                 |Below[ ]this[ ]line[ ]is[ ]a[ ]copy[ ]of[ ]the[ ]message
+                 |Message[ ]text[ ]follows:[ ]
+                 |Original[ ]message[ ]follows
+                 |The[ ]first[ ]\d+[ ]lines[ ]
+                 |Unsent[ ]Message[ ]below
+                 |Your[ ]message[ ]reads[ ][(]in[ ]part[)]:
+                )
+            }xi;
+
+            my $RxAddr = qr{(?:
+                     \A\s*
+                    |\A["].+["]\s*
+                    |\ARecipient:\s*
+                    |\A[ ]*Address:[ ]
+                    |addressed[ ]to[ ]
+                    |delivered[ ]to[ ]+
+                    |delivery[ ]failed:[ ]
+                    |Error-for:[ ]+
+                    |Failed[ ]Recipient:[ ]
+                    |generated[ ]from[ ]
+                    |Intended[ ]recipient:[ ]
+                    |Mailbox[ ]is[ ]full:[ ]
+                    |RCPT[ ]To:
+                    |Unknown[ ]User:[ ]
+                    |undeliverable[ ]to[ ]
+                    |Undeliverable[ ]Address:[ ]*
+                    |You[ ]sent[ ]mail[ ]to[ ]
+                    |Your[ ]message[ ]to[ ]
+                    )
+                [<]?([^\s\t\n\r@=]+[@][-.0-9A-Za-z]+[.][0-9A-Za-z]+)[>]?
+            }xi;
+
+            my $b = $dscontents->[ -1 ];
+            for my $e ( split( "\n", $$mbody ) ) {
+                # Get the recipient's email address and error messages.
+                last if $e =~ $RxRFC->{'endof'};
+                last if $e =~ $RxRFC->{'rfc822'};
+                last if $e =~ $RxEnd;
+                last if $e =~ m/\A[*]/;
+
+                next unless length $e;
+                next if $e =~ $RxSkip;
+
+                if( $e =~ $RxAddr ) {
+                    # May be an email address
+                    my $x = $b->{'recipient'} || '';
+                    my $y = Sisimai::Address->s3s4( $1 );
+
+                    if( length $x && $x ne $y ) {
+                        # There are multiple recipient addresses in the message body.
+                        push @$dscontents, Sisimai::MTA->DELIVERYSTATUS;
+                        $b = $dscontents->[ -1 ];
+                    }
+                    $b->{'recipient'} = $y;
+                    $b->{'agent'} = __PACKAGE__->smtpagent.'::Fallback';
+                    $recipients++;
+
+                } elsif( $e =~ m/[(]expanded[ ]from:[ ]([^@]+[@][^@]+)[)]/ ) {
+                    # (expanded from: neko@example.jp)
+                    $b->{'alias'} = Sisimai::Address->s3s4( $1 );
+
+                }
+                $b->{'diagnosis'} .= ' '.$e;
+            }
+        }
+    }
+
     return undef unless $recipients;
     require Sisimai::String;
     require Sisimai::RFC3463;
@@ -304,6 +440,7 @@ sub scan {
             $e->{'command'}   = '';
         }
         $e->{'status'} ||= Sisimai::RFC3463->getdsn( $e->{'diagnosis'} );
+        $e->{'command'}  = $1 if $e->{'diagnosis'} =~ $RxCmd;
 
         if( scalar @{ $mhead->{'received'} } ) {
             # Get localhost and remote host name from Received header.
