@@ -9,14 +9,18 @@ sub match {
     my $argvs = shift // return undef;
     my $regex = qr{(?>
          account[ ]is[ ](?:
-             over[ ]quota
+             exceeding[ ]their[ ]quota
+            |over[ ]quota
             |temporarily[ ]over[ ]quota
             )
         |delivery[ ]failed:[ ]over[ ]quota
         |disc[ ]quota[ ]exceeded
         |does[ ]not[ ]have[ ]enough[ ]space
         |exceeded[ ]storage[ ]allocation
+        |exceeding[ ]its[ ]mailbox[ ]quota
+        |full[ ]mailbox
         |is[ ]over[ ]quota[ ]temporarily
+        |is[ ]over[ ]disk[ ]quota
         |mail[ ](?:
              file[ ]size[ ]exceeds[ ]the[ ]maximum[ ]size[ ]allowed[ ]for[ ]mail[ ]delivery
             |quota[ ]exceeded
@@ -27,15 +31,18 @@ sub match {
             |has[ ]exceeded[ ]its[ ]disk[ ]space[ ]limit
             |is[ ]full
             |over[ ]quota
+            |quota[ ]usage[ ]exceeded
             |size[ ]limit[ ]exceeded
             )
         |maildir[ ](?:
              delivery[ ]failed:[ ](?:User|Domain)disk[ ]quota[ ]?.*[ ]exceeded
             |over[ ]quota
             )
+        |mailfolder[ ]is[ ]full
         |not[ ]enough[ ]storage[ ]space[ ]in
         |over[ ]the[ ]allowed[ ]quota
         |quota[ ]exceeded
+        |quota[ ]violation[ ]for
         |recipient[ ](?:
              reached[ ]disk[ ]quota
             |rejected:[ ]mailbox[ ]would[ ]exceed[ ]maximum[ ]allowed[ ]storage
