@@ -328,8 +328,9 @@ sub parse {
             }
 
         } elsif( $p =~ m/\A([0-2]\d):([0-5]\d):([0-5]\d)\z/ ||
-                 $p =~ m/\A(\d{1,2}):(\d{1,2}):(\d{1,2})\z/ ) {
+                 $p =~ m/\A(\d{1,2})[-:](\d{1,2})[-:](\d{1,2})\z/ ) {
             # Time; 12:34:56, 03:14:15, ...
+            # Arrival-Date: 2014-03-26 00-01-19
             if( $1 < 24 && $2 < 60 && $3 < 60 ) {
                 # Valid time format, maybe...
                 $v->{'T'} = sprintf( "%02d:%02d:%02d", $1, $2, $3 );
@@ -408,6 +409,7 @@ sub parse {
 
     # Check each piece
     if( grep { ! defined $_ } values %$v ) {
+        # Strange date format
         warn sprintf( " ***warning: Strange date format [%s]", $datestring );
         return undef;
     }
